#include "mcc_generated_files/mcc.h"

#define TIME_OFF -1         //タイマーを使用しない場合
#define TIME_UP 0           //タイムアップ
#define TIME_BASE_MAX 10     //ベースタイマカウント値
#define	CNT_INIT_MAX 10     //10ms×10 = 100ms
#define TIME_FILTER_MAX 1    //ベースタイマカウント値
#define TIME_LED_MAX 25     //250ms
#define TIME_LED_OFF 200    //2s
#define DI_NUM 4            //DIフィルタのサンプリング数

typedef struct{
    uint8_t    wp;
    uint8_t    buf[ DI_NUM ];
}FILT_DATA;
//--------------変数定義----------------------------
int16_t cnt10ms;
int16_t timfilter;   //DIフィルタ起動
int16_t timled;      //動作確認用LED
int16_t timledoff;  //チャタリング発生通知LEDをOFF
uint8_t CntInit;	//初期化時のみ使用
FILT_DATA DiData;
//--------------関数ﾌﾟﾛﾄﾀｲﾌﾟ宣言----------------------------
void mainApp(void);     //メイン関数内での処理
void TMR0_INT(void);    //ﾀｲﾏｰ0割り込みでの処理
void mainTimer(void);
void DiFilter(void);
void DiFilterInit(void);

/*　Main application　*/
void main(void)
{
    SYSTEM_Initialize();
    TMR0_SetInterruptHandler(TMR0_INT);
    INTERRUPT_GlobalInterruptEnable();
    INTERRUPT_PeripheralInterruptEnable();
    DiFilterInit(); //メイン処理前にDIを確定する

    while (1)
    {
        mainApp();
        mainTimer();
        DiFilter();
    }
}
/* main処理  */
void mainApp(void){
    
    if( timled == TIME_UP ){ //タイムアップしたか
        timled = TIME_LED_MAX;
        IO_RB6_Toggle();
    }
    
    if( timledoff == TIME_UP ){
        timledoff = TIME_OFF;
        IO_RA7_SetHigh(); //チャタリングが発生(OFF))
    }
}
/* TMR0オーバーフロー割り込みでの処理  */
void TMR0_INT(void){  
 
    ++cnt10ms;
}
/* タイマ管理関数 */
void mainTimer(void){
 
    if( cnt10ms >= TIME_BASE_MAX){ //規定回数オーバーフローしたか
        cnt10ms -= TIME_BASE_MAX;
        
        if( timfilter > TIME_UP ){
            --timfilter; //タイマを更新
        }
        
        if( timled > TIME_UP){
            --timled; //タイマを更新
        }
        
        if( timledoff > TIME_UP){
            --timledoff; //タイマを更新
        }
    }
}
// DIフィルタの初期化 //
void DiFilterInit(void){
	
    CntInit = CNT_INIT_MAX;
    while(CntInit > 0){  //0になるまでフィルタを実施
	DiFilter();         //DIフィルタ処理
	__delay_ms(10);     //10ms遅延させてDIフィルタ処理
        CntInit--;
	timfilter = TIME_UP;
    }    
}
//--------モード切替部のDIフィルタ----------------------------
void DiFilter(void){
    uint8_t i;
    bool boo = true;

    if( timfilter == TIME_UP ){
        timfilter = TIME_FILTER_MAX;

        DiData.buf[ DiData.wp ] = IO_RB0_GetValue();

        for( i = 1; i < DI_NUM; i++){
            if( DiData.buf[i-1] != DiData.buf[i] ){
                boo = false;
                IO_RA7_SetLow(); //チャタリングが発生
                timledoff = TIME_LED_OFF;
                break;
            }
        }

        if( boo ){ ////比較して一致したら値を採用
             IO_RA3_LAT = DiData.buf[ 0 ]; 
        }

        if( ++DiData.wp >= DI_NUM){
                DiData.wp = 0;
        }
    }
}
/* End of File */